@@ -53,6 +53,11 @@ Net::Jabber::Query::Oob - Jabber Query Out Of Bandwidth File Transfer Module
     $oob->SetURL("http://my.web.server.com/~me/pics/bobandme.jpg");
     $oob->SetDesc("Bob and Me at the Open Source conference");
 
+=head2 Defined functions
+
+    $test = $oob->DefinedURL();
+    $test = $oob->DefinedDesc();
+
 =head1 METHODS
 
 =head2 Retrieval functions
@@ -77,9 +82,15 @@ Net::Jabber::Query::Oob - Jabber Query Out Of Bandwidth File Transfer Module
 
   SetDesc(string) - sets the description for the file being sent Oob.
 
+=head2 Defined functions
+
+  DefinedURL() - returns 1 if <url/> is in the <iq/>, 0 otherwise.
+
+  DefinedDesc() - returns 1 if <desc/> is in the <iq/>, 0 otherwise.
+
 =head1 AUTHOR
 
-By Ryan Eatmon in May of 2000 for http://jabber.org..
+By Ryan Eatmon in July of 2000 for http://jabber.org..
 
 =head1 COPYRIGHT
 
@@ -91,9 +102,9 @@ it under the same terms as Perl itself.
 require 5.003;
 use strict;
 use Carp;
-use vars qw($VERSION);
+use vars qw($VERSION $AUTOLOAD %FUNCTIONS);
 
-$VERSION = "1.0013";
+$VERSION = "1.0017";
 
 sub new {
   my $proto = shift;
@@ -110,26 +121,34 @@ sub new {
 
 ##############################################################################
 #
-# GetURL - returns the url of the jabber:iq:oob
+# AUTOLOAD - This function calls the delegate with the appropriate function
+#            name and argument list.
 #
 ##############################################################################
-sub GetURL {
-  shift;
+sub AUTOLOAD {
+  my $parent = shift;
   my $self = shift;
-  return &Net::Jabber::GetXMLData("value",$self->{QUERY},"url","");
+  return if ($AUTOLOAD =~ /::DESTROY$/);
+  $AUTOLOAD =~ s/^.*:://;
+  my ($type,$value) = ($AUTOLOAD =~ /^(Get|Set|Defined)(.*)$/);
+  $type = "" unless defined($type);
+  my $treeName = "QUERY";
+
+  return &Net::Jabber::Get($parent,$self,$value,$treeName,$FUNCTIONS{get}->{$value},@_) if ($type eq "Get");
+  return &Net::Jabber::Set($parent,$self,$value,$treeName,$FUNCTIONS{set}->{$value},@_) if ($type eq "Set");
+  return &Net::Jabber::Defined($parent,$self,$value,$treeName,$FUNCTIONS{defined}->{$value},@_) if ($type eq "Defined");
+  &Net::Jabber::MissingFunction($parent,$AUTOLOAD);
 }
 
 
-##############################################################################
-#
-# GetDesc - returns the desc of the jabber:iq:oob
-#
-##############################################################################
-sub GetDesc {
-  shift;
-  my $self = shift;
-  return &Net::Jabber::GetXMLData("value",$self->{QUERY},"desc","");
-}
+$FUNCTIONS{get}->{URL}  = ["value","url",""];
+$FUNCTIONS{get}->{Desc} = ["value","desc",""];
+
+$FUNCTIONS{set}->{URL}  = ["single","url","*","",""];
+$FUNCTIONS{set}->{Desc} = ["single","desc","*","",""];
+
+$FUNCTIONS{defined}->{URL}  = ["existence","url",""];
+$FUNCTIONS{defined}->{Desc} = ["existence","desc",""];
 
 
 ##############################################################################
@@ -149,31 +168,4 @@ sub SetOob {
 }
 
 
-##############################################################################
-#
-# SetURL - sets the url in the jabber:iq:oob
-#
-##############################################################################
-sub SetURL {
-  shift;
-  my $self = shift;
-  my ($url) = @_;
-  &Net::Jabber::SetXMLData("single",$self->{QUERY},"url",$url,{});
-}
-
-
-##############################################################################
-#
-# SetDesc - sets the desc in the jabber:iq:oob
-#
-##############################################################################
-sub SetDesc {
-  shift;
-  my $self = shift;
-  my ($desc) = @_;
-  
-  &Net::Jabber::SetXMLData("single",$self->{QUERY},"desc",$desc,{});
-}
-
-
 1;
